from fastapi import APIRouter, Depends, Request, status
from fastapi.responses import RedirectResponse
from fastapi.templating import Jinja2Templates
from sqlalchemy.orm import Session

from app import crud
from app.config import settings
from app.dependencies import get_db
from app.constants import TEMPLATES_DIR

router = APIRouter()
templates = Jinja2Templates(directory=TEMPLATES_DIR)


@router.get("/")
def index(request: Request):
    is_authenticated = request.state.user is not None
    if not is_authenticated:
        return RedirectResponse(url="/login", status_code=status.HTTP_302_FOUND)
    return RedirectResponse(url="/storage", status_code=status.HTTP_302_FOUND)


@router.get("/login")
def login(request: Request):
    if request.state.user is not None:
        return RedirectResponse(url="/storage", status_code=status.HTTP_302_FOUND)
    return templates.TemplateResponse(
        "login.html",
        {
            "request": request,
            "is_authenticated": False,
            "title": f"{settings.APP_TITLE} - Login",
        },
    )


@router.get("/storage")
def get_storage(request: Request, db: Session = Depends(get_db)):
    if request.state.user is None:
        return RedirectResponse(url="/login", status_code=status.HTTP_302_FOUND)

    chests = crud.chest.get_random(db=db, num=10)
    return templates.TemplateResponse(
        "storage.html",
        {
            "title": f"{settings.APP_TITLE} - Storage",
            "request": request,
            "current_user": request.state.user,
            "chests": chests,
        },
    )


@router.get("/storage/chests/{uid}")
def get_chest(request: Request, uid: str, db: Session = Depends(get_db)):
    if request.state.user is None:
        return RedirectResponse(url="/login", status_code=status.HTTP_302_FOUND)

    chest = crud.chest.get_by_uid(db=db, uid=uid)
    if not chest:
        return RedirectResponse(url="/storage?chest_not_found=narr!")
    return templates.TemplateResponse(
        "chest.html",
        {
            "title": f"{settings.APP_TITLE} - Chest {uid[:8]}",
            "request": request,
            "current_user": request.state.user,
            "chest_id": chest.id,
        },
    )
